==> portmanager-0.3.4.tar.gz: portmanager-0.3.4/libMGPM/src/MGPMrShowLeaves.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrShowLeaves( structProperty* property )
{
	FILE*	pHandle			= NULL;
	char	exact[]			= "exact";
	char	id[]			= "MGPMrShowLeaves";
	char*	buffer			= NULL;
	char*	command			= NULL;
	char*	leafPortDir		= NULL;
	char*	leafPortName		= NULL;
	int	bufferIdx		= 0;
	int	bufferSize		= 0xffff;
	int	installedPortsDbIDX	= 0;
	int	installedPortsDbQTY	= 0;

	command		= (char*)calloc( bufferSize, 1 );
	buffer		= (char*)calloc( bufferSize, 1 );
	leafPortDir	= (char*)calloc( bufferSize, 1 );
	leafPortName	= (char*)calloc( bufferSize, 1 );

	if( ( property->installedPortsDb = MGdbOpen( property->installedPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->installedPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->availableDependenciesDb = MGdbOpen( property->availableDependenciesDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout, "Ports with no ports depending on them to run or build AKA \"leaf ports.\"\n" );
	fprintf( stdout, "These ports may be deinstalled because no other ports rely on them.\"\n" );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
	installedPortsDbIDX	= 1;
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		strcpy( leafPortDir, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir ) );
		strcpy( leafPortName, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortName ) );
		MGdbGoTop( property->availableDependenciesDb );
		if( MGdbSeek( property->availableDependenciesDb,
			property->fieldAvailableDependenciesDbDependencyDir,
			leafPortDir,
			property->fieldAvailableDependenciesDbDependencyDir,
			exact ) == NULL )
		{
			fprintf( stdout, "%-30s %-30s ", leafPortName, leafPortDir );
			while( fflush( stdout ) );
			strcpy( command, "grep COMMENT " );
			strcat( command, PORTSDIR );
			strcat( command, leafPortDir );
			strcat( command, "/Makefile" );
			pHandle		= popen( command, "r" );
			buffer[0]	= 0;
			fread( buffer, bufferSize - 1, 1, pHandle );
			pclose( pHandle );
			bufferIdx	= 0;
			while( bufferIdx < bufferSize )
			{
				if( buffer[bufferIdx]	== LINEFEED )
				{
					buffer[bufferIdx]	= 0;
					break;
				}
				bufferIdx++;
			}
			fprintf( stdout, "%-25s\n", buffer+9 );
			while( fflush( stdout ) );
		}
		installedPortsDbIDX++;
	}
	fprintf( stdout, "%s\n", DOUBLE_LINES );
	while( fflush( stdout ) );
	free( buffer );
	free( command );
	free( leafPortDir );
	free( leafPortName );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->availableDependenciesDb );
	return(0);
}

==> portmanager-0.4.1.tar.gz: portmanager-0.4.1/libMGPM/src/MGPMrShowLeaves.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrShowLeaves( structProperty* property )
{
	FILE*	pHandle			= NULL;
	char	exact[]			= "exact";
	char	id[]			= "MGPMrShowLeaves";
	char*	buffer			= NULL;
	char*	command			= NULL;
	char*	leafPortDir		= NULL;
	char*	leafPortName		= NULL;
	int	bufferIdx		= 0;
	int	bufferSize		= 0xffff;
	int	installedPortsDbIDX	= 0;
	int	installedPortsDbQTY	= 0;

	command		= (char*)calloc( bufferSize, 1 );
	buffer		= (char*)calloc( bufferSize, 1 );
	leafPortDir	= (char*)calloc( bufferSize, 1 );
	leafPortName	= (char*)calloc( bufferSize, 1 );

	if( ( property->installedPortsDb = MGdbOpen( property->installedPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->installedPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->availableDependenciesDb = MGdbOpen( property->availableDependenciesDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout, "Ports with no ports depending on them to run or build AKA \"leaf ports.\"\n" );
	fprintf( stdout, "These ports may be deinstalled because no other ports rely on them.\"\n" );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
	installedPortsDbIDX	= 1;
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		MGmStrcpy( leafPortDir, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir ) );
		MGmStrcpy( leafPortName, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortName ) );
		MGdbGoTop( property->availableDependenciesDb );
		if( MGdbSeek( property->availableDependenciesDb,
			property->fieldAvailableDependenciesDbDependencyDir,
			leafPortDir,
			property->fieldAvailableDependenciesDbDependencyDir,
			exact ) == NULL )
		{
			fprintf( stdout, "%-30s %-30s ", leafPortName, leafPortDir );
			while( fflush( stdout ) );
			MGmStrcpy( command, "grep COMMENT " );
			MGmStrcat( command, PORTSDIR );
			MGmStrcat( command, leafPortDir );
			MGmStrcat( command, "/Makefile" );
			pHandle		= popen( command, "r" );
			buffer[0]	= 0;
			fread( buffer, bufferSize - 1, 1, pHandle );
			pclose( pHandle );
			bufferIdx	= 0;
			while( bufferIdx < bufferSize )
			{
				if( buffer[bufferIdx]	== LINEFEED )
				{
					buffer[bufferIdx]	= 0;
					break;
				}
				bufferIdx++;
			}
			fprintf( stdout, "%-25s\n", buffer+9 );
			while( fflush( stdout ) );
		}
		installedPortsDbIDX++;
	}
	fprintf( stdout, "%s\n", DOUBLE_LINES );
	while( fflush( stdout ) );
	free( buffer );
	free( command );
	free( leafPortDir );
	free( leafPortName );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->availableDependenciesDb );
	return(0);
}
